#
# Copyright (C) 2008-2009 Ido Abramovich <ido.deluge@gmail.com>
# Copyright (C) 2009 Andrew Resch <andrewresch@gmail.com>
#
# This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
# the additional special exception to link portions of this program with the OpenSSL library.
# See LICENSE for more details.
#

import deluge.component as component

from . import BaseCommand


class Command(BaseCommand):
    """Exit the client"""

    aliases = ['exit']
    interactive_only = True

    def handle(self, options):
        component.get('ConsoleUI').quit()
